@@ -21,11 +21,11 @@
 #include <kapplication.h>
 #include <kdebug.h>
 #include <klocale.h>
+#include <kdialogbase.h>
 #include <kmessagebox.h>
 #include <kprocess.h>
 
 /* QT INCLUDES */
-#include <qapplication.h>
 #include <qcstring.h>
 #include <qdatastream.h>
 #include <qfileinfo.h>
@@ -47,7 +47,9 @@
 #include "kommanderwidget.h"
 #include "specials.h"
 #include "specialinformation.h"
-
+#include "parser.h"
+#include "parserdata.h"
+#include "kommanderwindow.h"
 
 KommanderWidget::KommanderWidget(QObject *a_thisObject)
 {
@@ -72,7 +74,7 @@ QStringList KommanderWidget::associatedText() const
 
 bool KommanderWidget::hasAssociatedText()
 {
-  int index = (states().findIndex( currentState()));
+  int index = states().findIndex(currentState());
   if (index == -1 || m_associatedText[index].isEmpty())
     return false;
   return true;
@@ -122,6 +124,22 @@ QString KommanderWidget::evalAssociatedText() // expands and returns associated
 
 QString KommanderWidget::evalAssociatedText(const QString& a_text)
 {
+  /* New internal parser is used if global flag is set */
+  if ((KommanderWidget::useInternalParser && !a_text.startsWith("#!")) || a_text.startsWith("#!kommander"))
+  {
+    Parser p(internalParserData());
+    p.setString(a_text);
+    p.setWidget(this);
+    if (!p.parse())
+    {
+      // FIXME add widget's name to KommanderWidget class      
+      printError(i18n("Line %1: %2.\n").arg(p.errorLine()+1).arg(p.errorMessage()));
+    }
+    return QString::null;
+  }
+  
+  /* Old macro-only parser is implemented below  */
+  
   QString evalText;
   
   int pos = 0, baseTextLength = a_text.length();
@@ -199,9 +217,16 @@ QString KommanderWidget::evalAssociatedText(const QString& a_text)
           evalText += evalArrayFunction(function, args);
           break;
         case Group::String:
-          evalText += evalStringFunction(function, args);
+          evalText += Parser::function(internalParserData(), "str_" + function, args);
+          break;
         case Group::File:
-          evalText += evalFileFunction(function, args);
+          evalText += Parser::function(internalParserData(), "file_" + function, args);
+          break;
+        case Group::Message:
+          evalText += Parser::function(internalParserData(), "message_" + function, args);
+          break;
+        case Group::Input:
+          evalText += Parser::function(internalParserData(), "input_" + function, args);
           break;
         default:
           return QString::null;
@@ -254,7 +279,7 @@ QString KommanderWidget::DCOPQuery(const QStringList& a_query)
     else if (argTypes[i] == "double")
       byteDataStream << a_query[i+3].toDouble();
     else if (argTypes[i] == "bool")
-      byteDataStream << (bool)(a_query[i+3] != "false" && a_query[i+3] != "FALSE" && a_query[i] != "0");
+      byteDataStream << (bool)(a_query[i+3] != "false" && a_query[i+3] != "false" && a_query[i+3] != "0");
     else if (argTypes[i] == "QStringList")
       if (a_query[i+3].find('\n') != -1)
         byteDataStream << QStringList::split("\n", a_query[i+3], true);
@@ -284,6 +309,12 @@ QString KommanderWidget::DCOPQuery(const QStringList& a_query)
     byteReplyStream >> i;
     return QString::number(i);
   }
+  else if(replyType == "bool")
+  {
+    bool b;
+    byteReplyStream >> b;
+    return QString::number(b);
+  }
   else if (replyType == "QStringList")
   {
     QStringList text;
@@ -318,9 +349,9 @@ QString KommanderWidget::localDCOPQuery(const QString function, const QString& a
   pArgs.append(function);
   pArgs.append(arg1);
   pArgs.append(arg2);
-  if (arg3 != QString::null)
+  if (!arg3.isNull())
     pArgs.append(arg3);
-  if (arg4 != QString::null)
+  if (!arg4.isNull())
     pArgs.append(arg4);
   return DCOPQuery(pArgs);
 }
@@ -355,8 +386,26 @@ void KommanderWidget::printError(const QString& a_error) const
 {
   if (showErrors) 
   {
-    KMessageBox::error(0, i18n("<qt>Error in widget <b>%1</b>:<p><i>%2</i></qt>")
-       .arg(QString(m_thisObject->name())).arg(a_error)); 
+    KDialogBase* dialog = new KDialogBase("Error", KDialogBase::Yes | KDialogBase::No | KDialogBase::Cancel,
+                KDialogBase::Yes, KDialogBase::No, 0, 0, true, false, 
+                i18n("Continue"), i18n("Continue && Ignore Next Errors"), i18n("Stop"));
+    switch (KMessageBox::createKMessageBox(dialog, QMessageBox::Warning, 
+                i18n("<qt>Error in widget <b>%1</b>:<p><i>%2</i></qt>").arg(QString(m_thisObject->name()))
+                    .arg(a_error), QStringList(), QString::null, 0, 0))
+    {
+      case KDialogBase::No:
+        showErrors = false;
+      case KDialogBase::Yes:
+        break;
+      case KDialogBase::Cancel:
+        if (parentDialog()->inherits("QDialog"))
+        {
+          parentDialog()->close();
+          exit(-1);
+        }
+        else if (parentDialog()->inherits("QMainWindow"))
+          kapp->quit();
+    }
   }
   else 
   {
@@ -453,7 +502,7 @@ QString KommanderWidget::parseQuotes(const QString& s) const
 {
   if (s[0] == s[s.length()-1] && (s[0] == '\'' || s[0] == '\"'))
   {
-    QChar buf[s.length()];
+    QMemArray<QChar> buf(s.length());
     int start = 0;
     int end = s.length() - 1;
     for (int i=1; i<end; i++)
@@ -480,6 +529,17 @@ QString KommanderWidget::parseQuotes(const QString& s) const
   else return s;
 }
 
+bool KommanderWidget::isWidget(const QString& a_name) const
+{
+  return parseWidget(a_name);  
+}
+
+KommanderWidget* KommanderWidget::widgetByName(const QString& a_name) const
+{
+  return parseWidget(a_name);
+}
+
+
 KommanderWidget* KommanderWidget::parseWidget(const QString& widgetName) const
 {
   if (QString(parentDialog()->name()) == widgetName) 
@@ -516,12 +576,12 @@ QStringList KommanderWidget::parseFunction(const QString group, const QString& f
     printError(i18n("Not enough arguments for \'%1\' (%2 instead of %3).<p>"
        "Correct syntax is: %4")
         .arg(function).arg(args.count() + extraArg).arg(SpecialInformation::minArg(gname, fname))
-        .arg(SpecialInformation::longPrototype(gname, fname)));
+        .arg(SpecialInformation::prototype(gname, fname, SpecialFunction::ShowArgumentNames)));
   else if ((int)args.count() + extraArg > SpecialInformation::maxArg(gname, fname))
     printError(i18n("Too many arguments for \'%1\' (%2 instead of %3).<p>"
        "Correct syntax is: %4")
       .arg(function).arg(args.count() + extraArg).arg(SpecialInformation::maxArg(gname, fname))
-      .arg(SpecialInformation::longPrototype(gname, fname)));
+      .arg(SpecialInformation::prototype(gname, fname, SpecialFunction::ShowArgumentNames)));
   else 
     success = true;
   ok = success;
@@ -566,16 +626,16 @@ QString KommanderWidget::substituteVariable(QString text, QString variable, QStr
 
 
 
-QObject* KommanderWidget::parentDialog() const
+QWidget* KommanderWidget::parentDialog() const
 {
   QObject *superParent = m_thisObject;
   while (superParent->parent())
   {
     superParent = superParent->parent();
-    if (superParent->inherits("Dialog"))
+    if (superParent->inherits("QDialog") || superParent->inherits("QMainWindow"))
       break;
   }
-  return superParent;
+  return (QWidget*)superParent;
 }
 
 
@@ -594,13 +654,62 @@ void KommanderWidget::setGlobal(const QString& variableName, const QString& valu
   m_globals.insert(variableName, value); 
 }  
 
-QString KommanderWidget::handleDCOP(const int, const QStringList&)
+QString KommanderWidget::handleDCOP(const int function, const QStringList& args)
 {
+  QWidget* current = dynamic_cast<QWidget*>(m_thisObject);
+  if (!current) 
+    return QString::null;
+  switch(function) {
+    case DCOP::setEnabled:
+      current->setEnabled(args[0] != "false" && args[0] != "0");
+      break;
+    case DCOP::setVisible:
+      current->setShown(args[0] != "false" && args[0] != "0");
+      break;
+    case DCOP::type:
+      return current->className();      
+    case DCOP::children:
+    {
+      QStringList matching;
+      QObjectList* widgets = current->queryList("QWidget", 0, false, args.count() == 0 || args[0] != "false");
+      for (QObject* w = widgets->first(); w; w = widgets->next())
+        if (w->name() && (dynamic_cast<KommanderWidget*>(w)))
+            matching.append(w->name());
+      return matching.join("\n");  
+    }  
+  }
   return QString::null;
 }
 
+bool KommanderWidget::isFunctionSupported(int f)
+{
+  return f == DCOP::setEnabled || f == DCOP::setVisible || f == DCOP::children || f == DCOP::type;
+}
+
+bool KommanderWidget::isCommonFunction(int f)
+{
+  return f == DCOP::setEnabled || f == DCOP::setVisible || f == DCOP::children || f == DCOP::type;
+}
+
+ParserData* KommanderWidget::internalParserData()
+{
+  return m_parserData;
+}
+
+QString KommanderWidget::fileName()
+{
+  KommanderWindow* window = dynamic_cast<KommanderWindow*>(parentDialog());
+  if (window)
+    return QString(window->fileName());
+  else
+    return QString::null;
+}  
+
 
 bool KommanderWidget::inEditor = false;
 bool KommanderWidget::showErrors = true;
+bool KommanderWidget::useInternalParser = false;
 QMap<QString, QString> KommanderWidget::m_globals;
 QMap<QString, QMap<QString, QString> > KommanderWidget::m_arrays;
+ParserData* KommanderWidget::m_parserData = new ParserData;
+
