@@ -46,7 +46,7 @@ QString ProgressBar::currentState() const
 
 bool ProgressBar::isKommanderWidget() const
 {
-  return TRUE;
+  return true;
 }
 
 QStringList ProgressBar::associatedText() const
@@ -100,7 +100,7 @@ QString ProgressBar::handleDCOP(int function, const QStringList& args)
       setTotalSteps(args[0].toUInt());
       break;
     default:
-      break;
+      return KommanderWidget::handleDCOP(function, args);
   }
   return QString::null;
 }
