@@ -26,9 +26,10 @@
 
 /* OTHER INCLUDES */
 #include <kommanderwidget.h>
+#include "kommander_export.h"
 
 class QShowEvent;
-class TextEdit: public KTextEdit, public KommanderWidget
+class KOMMANDER_EXPORT TextEdit: public KTextEdit, public KommanderWidget
 {
   Q_OBJECT Q_PROPERTY(QString populationText READ populationText WRITE setPopulationText DESIGNABLE false)
   Q_PROPERTY(QStringList associations READ associatedText WRITE setAssociatedText DESIGNABLE false)
