@@ -14,13 +14,16 @@
  ***************************************************************************/
 
 //qt includes
+#include <qeventloop.h>
 #include <qstring.h>
+#include <qpoint.h>
 #include <qregexp.h>
 #include <qcstring.h>
 #include <qstringlist.h>
 #include <qstrlist.h>
 #include <qdatetime.h>
 #include <qfile.h>
+#include <qtextcodec.h>
 #include <qvaluelist.h>
 #include <qvaluestack.h>
 
@@ -39,12 +42,9 @@
 #include "quantaview.h"
 #include "quantacommon.h"
 #include "document.h"
-#include "quanta.h"
 #include "qextfileinfo.h"
-#ifdef BUILD_KAFKAPART
 #include "kafkacommon.h"
 #include "undoredo.h"
-#endif
 #include "dtds.h"
 #include "structtreetag.h"
 
@@ -56,7 +56,9 @@
 #include <kdirwatch.h>
 #include <kiconloader.h>
 #include <klocale.h>
+#include <ktexteditor/document.h>
 #include <ktexteditor/editinterface.h>
+#include <ktexteditor/encodinginterface.h>
 #include <ktexteditor/viewcursorinterface.h>
 
 extern GroupElementMapList globalGroupMap;
@@ -177,7 +179,7 @@ Node *Parser::parseArea(int startLine, int startCol, int endLine, int endCol, No
         bool insideSingleQuotes = false;
         bool insideDoubleQuotes = false;
         //find the matching ">" in the document
-        while (line <= endLine && openNum > 0)
+        while (line <= endLine && openNum > 0 && !firstOpenFound)
         {
           textLine = ParserCommon::getLine(write, line, endLine, endCol);
           uint textLineLen = textLine.length();
@@ -195,11 +197,15 @@ Node *Parser::parseArea(int startLine, int startCol, int endLine, int endCol, No
                 if (textLine[i] == '<')
                 {
                   openNum++;
-                  if (!firstOpenFound)
+                  if (!firstOpenFound &&
+                      (i < textLineLen -1 && (textLine[i + 1] == '/' || textLine[i + 1].isLetter()) ||
+                       i == textLineLen -1)
+                     )
                   {
                     firstStartCol = i;
                     firstStartLine = line;
                     firstOpenFound = true;
+                    break;
                   }
                 } else
                 if (textLine[i] == '>') openNum--;
@@ -315,7 +321,7 @@ Node *Parser::parseArea(int startLine, int startCol, int endLine, int endCol, No
           {
             QString searchFor = (m_dtd->caseSensitive)?tag->name:tag->name.upper();
             searchFor.remove('/');
-            if ( qTag->stoppingTags.contains( searchFor ) )
+            if ( qTag->stoppingTags.contains(searchFor))
             {
               parentNode->tag->closingMissing = true; //parent is single...
               goUp = true;
@@ -432,6 +438,7 @@ Node *Parser::parseArea(int startLine, int startCol, int endLine, int endCol, No
         line++;
         col = 0;
         textLine = ParserCommon::getLine(write, line, endLine, endCol);
+        //kdDebug(24000) << "Line " << line << endl;
       }
 
     }
@@ -449,6 +456,9 @@ Node *Parser::parseArea(int startLine, int startCol, int endLine, int endCol, No
     if (ec == -1)
         ec = 0;
     AreaStruct area(el, ec, endLine, endCol);
+#ifdef DEBUG_PARSER
+    kdDebug(24000) << "Calling cleanGroups from Parser::parseArea" << endl;
+#endif
     cleanGroups();
     m_saParser->setParsingEnabled(true);
     currentNode = m_saParser->parseArea(area, "", "", parentNode, true, true); //TODO: don't parse in detail here
@@ -491,12 +501,10 @@ Node *Parser::parse(Document *w, bool force)
 {
   QTime t;
   t.start();
- QuantaView *view = ViewManager::ref()->activeView();
-#ifdef BUILD_KAFKAPART
+  QuantaView *view = ViewManager::ref()->activeView();
   //If VPL is loaded, there shouldn't be any rebuild
   if(view && view->hadLastFocus() == QuantaView::VPLFocus && !force)
     return m_node;
-#endif
 
   if(!m_parsingEnabled && !force)
     return baseNode;
@@ -515,6 +523,7 @@ Node *Parser::parse(Document *w, bool force)
   Node *lastNode;
   write = w;
   m_dtd = w->defaultDTD();
+  w->resetDTEPs();
   maxLines = w->editIf->numLines() - 1;
   parsingEnabled = true;
   nodeNum = 0;
@@ -536,12 +545,10 @@ Node *Parser::parse(Document *w, bool force)
  baseNode = m_node;
  m_saParser->init(m_node, w);
 
-#ifdef BUILD_KAFKAPART
  //We need to reload Kafka to refresh the DOM::Node->Node links.
  //FIXME: make a function which only update the DOM::Node->Node links.
   if (view)
     view->reloadVPLView(true);
-#endif
 
  emit nodeTreeChanged();
  if (saParserEnabled)
@@ -684,7 +691,6 @@ Node *Parser::nodeAt(int line, int col, bool findDeepest, bool exact)
 }
 void Parser::logReparse(NodeModifsSet *modifs, Document *w)
 {
-#ifdef BUILD_KAFKAPART
   NodeModif *modif;
   if (baseNode)
   {
@@ -704,10 +710,6 @@ void Parser::logReparse(NodeModifsSet *modifs, Document *w)
   modif->setType(NodeModif::NodeTreeAdded);
   modifs->addNodeModif(modif);
   w->docUndoRedo->addNewModifsSet(modifs, undoRedo::SourceModif);
-#else
-  Q_UNUSED(modifs);
-  Q_UNUSED(w);
-#endif
 }
 
 bool Parser::invalidArea(Document *w, AreaStruct &area, Node **firstNode, Node **lastNode)
@@ -827,9 +829,7 @@ void Parser::deleteNodes(Node *firstNode, Node *lastNode, NodeModifsSet *modifs)
   int i, j;
   Node *node = firstNode;
   bool closesPrevious = false;
-#ifdef BUILD_KAFKAPART
   NodeModif *modif;
-#endif
   //delete all the nodes between the firstNode and lastNode
   while (node && node != lastNode )
   {
@@ -853,7 +853,6 @@ void Parser::deleteNodes(Node *firstNode, Node *lastNode, NodeModifsSet *modifs)
     if (next && next->closesPrevious)
       next->closesPrevious = false;
 
-#ifdef BUILD_KAFKAPART
     modif = new NodeModif();
     modif->setType(NodeModif::NodeRemoved);
     modif->setLocation(kafkaCommon::getLocation(node));
@@ -866,10 +865,6 @@ void Parser::deleteNodes(Node *firstNode, Node *lastNode, NodeModifsSet *modifs)
     node->prev = 0L;
     node->detachNode();
     modif->setNode(node);
-#else
-    delete node;
-    nodeNum--;
-#endif
     node = 0L;
     i = 0;
     j = 0;
@@ -975,20 +970,15 @@ void Parser::deleteNodes(Node *firstNode, Node *lastNode, NodeModifsSet *modifs)
 
       }
     }
-#ifdef BUILD_KAFKAPART
     modif->setChildrenMovedUp(i);
     modif->setNeighboursMovedDown(j);
     modifs->addNodeModif(modif);
-#endif
     node = nextNode;
 
  //   kdDebug(24000)<< "Node removed!" << endl;
 //    coutTree(m_node, 2);
   }
 //  coutTree(m_node, 2);
-#ifndef BUILD_KAFKAPART
-  Q_UNUSED(modifs);
-#endif
 }
 
 Node *Parser::rebuild(Document *w)
@@ -998,27 +988,23 @@ Node *Parser::rebuild(Document *w)
  t.start();
  bool saParserEnabled = m_saParser->parsingEnabled();
 
-#ifdef BUILD_KAFKAPART
   //If VPL is loaded, there shouldn't be any rebuild
   if(ViewManager::ref()->activeView()->hadLastFocus() == QuantaView::VPLFocus)
     return m_node;
 
  NodeModifsSet *modifs = new NodeModifsSet();
  NodeModif *modif;
-#endif
 
-#ifdef BUILD_KAFKAPART
+
 // kdDebug(24000)<< "Node *Parser::rebuild()" << endl;
- modifs->setIsModified(w->isModified());
-#endif
+ modifs->setIsModifiedAfter(w->isModified());
+
  //**kdDebug(24000)<< "************* Begin User Modification *****************" << endl;
   //debug!
   //coutTree(m_node, 2);//*/
  if (w != write || !m_node) //the document has changed or the top node does not exists => parse completely
  {
-#ifdef BUILD_KAFKAPART
   logReparse(modifs, w);
-#endif
    return parse(w);
  } else
  {
@@ -1037,20 +1023,15 @@ Node *Parser::rebuild(Document *w)
         (area.eLine < area.bLine || (area.eLine == area.bLine && area.eCol <= area.bCol)) //something strange has happened, like moving text with D&D inside the editor
       )
    {
-#ifdef BUILD_KAFKAPART
      logReparse(modifs, w);
-#endif
      m_saParser->setParsingEnabled(saParserEnabled);
      return parse(w);
    }
 
    kdDebug(24000) << QString("Invalid area: %1,%2,%3,%4").arg(area.bLine).arg(area.bCol).arg(area.eLine).arg(area.eCol) << "\n";
 
-#ifdef BUILD_KAFKAPART
    deleteNodes(firstNode->nextSibling(), lastNode, modifs);
-#else
-   deleteNodes(firstNode->nextSibling(), lastNode, 0L);
-#endif
+
 
    firstNode->child = 0L;
    Node *lastInserted = 0L;
@@ -1062,7 +1043,6 @@ Node *Parser::rebuild(Document *w)
    }
    node = parseArea(area.bLine, area.bCol, area.eLine, area.eCol, &lastInserted, firstNode);
 
-#ifdef BUILD_KAFKAPART
    Node *swapNode = firstNode->nextSibling();
    Node *p = (lastInserted)?lastInserted->nextSibling():lastInserted;
    while(swapNode != p)
@@ -1073,7 +1053,6 @@ Node *Parser::rebuild(Document *w)
       modifs->addNodeModif(modif);
       swapNode = swapNode->nextSibling();
    }
-#endif
    //another stange case: the parsed area contains a special area without end
    if (!node)
    {
@@ -1087,9 +1066,7 @@ Node *Parser::rebuild(Document *w)
      delete lastNode;
      nodeNum--;
      lastNode = 0L;
-#ifdef BUILD_KAFKAPART
      logReparse(modifs, w);
-#endif
      m_saParser->setParsingEnabled(saParserEnabled);
      return parse(w);
    }
@@ -1109,9 +1086,7 @@ Node *Parser::rebuild(Document *w)
         lastNode->parent = lastInserted->parent;
         lastInserted->tag->beginPos(area.bLine, area.bCol);
         lastNode->tag->endPos(area.eLine, area.eCol);
-#ifdef BUILD_KAFKAPART
         Tag *_tag = new Tag(*(lastNode->tag));
-#endif
         lastNode->tag->setTagPosition(area);
         QString s = write->text(area);
         lastNode->tag->setStr(s);
@@ -1125,7 +1100,7 @@ Node *Parser::rebuild(Document *w)
         if (lastInserted->parent && lastInserted->parent->child == lastInserted)
             //lastInserted->parent->child = lastInserted->next; lastInserted has no next!
            lastInserted->parent->child = lastNode;
-#ifdef BUILD_KAFKAPART
+
         //here, lastNode is at the pos of lastInserted.
         modif = new NodeModif();
         modif->setType(NodeModif::NodeRemoved);
@@ -1147,11 +1122,7 @@ Node *Parser::rebuild(Document *w)
         modif->setLocation(kafkaCommon::getLocation(lastNode));
         modif->setTag(_tag);
         modifs->addNodeModif(modif);
-#else
-        lastInserted->removeAll = false;
-        delete lastInserted;
-        nodeNum--;
-#endif
+
         lastInserted = lastNode;
         lastNode = lastNode->nextNotChild();
       }
@@ -1248,9 +1219,7 @@ Node *Parser::rebuild(Document *w)
 /*   kdDebug(24000)<< "END"<< endl;
    coutTree(baseNode,  2);
    kdDebug(24000)<< "************* End User Modification *****************" << endl;*/
-#ifdef BUILD_KAFKAPART
    w->docUndoRedo->addNewModifsSet(modifs, undoRedo::SourceModif);
-#endif
  }
   kdDebug(24000) << "Rebuild: " << t.elapsed() << " ms \n";
 /*
@@ -1269,9 +1238,13 @@ Node *Parser::rebuild(Document *w)
 
 void Parser::clearGroups()
 {
+#ifdef DEBUG_PARSER
+  kdDebug(24000) << "clearGroups " << endl;
+#endif
   GroupElementMapList::Iterator it;
   GroupElementList::Iterator elementIt;
   GroupElementList *list;
+  int count = 0;
   for (it = globalGroupMap.begin(); it != globalGroupMap.end(); ++it)
   {
     list = & it.data();
@@ -1280,17 +1253,32 @@ void Parser::clearGroups()
     elementIt = list->begin();
     while (elementIt != list->end())
     {
-       if (!(*elementIt).deleted)
-       {
-          Node *n = (*elementIt).node;
-          n->groupElementLists.clear();
-          n->groupTag = 0L;
-          delete (*elementIt).tag;
-       }
-       elementIt = list->erase(elementIt);
+      GroupElement *groupElement = (*elementIt);
+#ifdef DEBUG_PARSER
+      kdDebug(24001) << "GroupElement deleted: " <<groupElement << " "<< groupElement->tag->area().bLine << " " << groupElement->tag->area().bCol << " "<< groupElement->tag->area().eLine << " "<< groupElement->tag->area().eCol << " " << groupElement->tag->tagStr() << " " << groupElement->type << endl;
+#endif
+      //kdDebug(24000) << "usertagcount: " << groupElement->tag->write()->userTagList.count() << endl;
+      groupElement->tag->write()->userTagList.remove(groupElement->tag->name.lower());
+      if (!groupElement->deleted)
+      {
+        Node *n = groupElement->node;
+        n->m_groupElements.clear();
+      }
+      groupElement->group = 0L;
+      delete groupElement->tag;
+      groupElement->tag = 0L;
+      elementIt = list->erase(elementIt);
+      delete groupElement;
+      groupElement = 0L;
+      count++;
     }
   }
+#ifdef DEBUG_PARSER
+      kdDebug(24000) << count << " GroupElement deleted (clearGroups)." << endl;
+#endif
   globalGroupMap.clear();
+  clearIncludedGroupElements();
+    
   ParserCommon::includedFiles.clear();
   ParserCommon::includedFilesDTD.clear();
   delete ParserCommon::includeWatch;
@@ -1301,10 +1289,13 @@ void Parser::clearGroups()
 
 void Parser::cleanGroups()
 {
+#ifdef DEBUG_PARSER
   kdDebug(24000) << "cleanGroups " << endl;
+#endif
   GroupElementMapList::Iterator it;
   GroupElementList::Iterator elementIt;
   GroupElementList *list;
+  int count = 0;
   for (it = globalGroupMap.begin(); it != globalGroupMap.end(); ++it)
   {
     list = & it.data();
@@ -1313,32 +1304,42 @@ void Parser::cleanGroups()
     elementIt = list->begin();
     while (elementIt != list->end())
     {
-       if ((*elementIt).deleted)
+      GroupElement *groupElement = (*elementIt);
+      if (groupElement->deleted)
        {
+#ifdef DEBUG_PARSER
+         kdDebug(24001) << "GroupElement deleted: " <<groupElement << " "<< groupElement->tag->area().bLine << " " << groupElement->tag->area().bCol << " "<< groupElement->tag->area().eLine << " "<< groupElement->tag->area().eCol << " " << groupElement->tag->tagStr() << " " << groupElement->type << endl;
+#endif
+         groupElement->tag->write()->userTagList.remove(groupElement->tag->name.lower());
+         groupElement->group = 0L;
+         delete groupElement->tag;
+         groupElement->tag = 0L;
          elementIt = list->erase(elementIt);
+         delete groupElement;
+         groupElement = 0L;
+         count++;
        } else
        {
          ++elementIt;
        }
     }
   }
+#ifdef DEBUG_PARSER
+      kdDebug(24000) << count << "GroupElement deleted (cleanGroups)." << endl;
+#endif
   if (m_parseIncludedFiles)
   {
       delete ParserCommon::includeWatch;
       ParserCommon::includeWatch = new KDirWatch();
       connect(ParserCommon::includeWatch, SIGNAL(dirty(const QString&)), SLOT(slotIncludedFileChanged(const QString&)));
       parseIncludedFiles();
-      m_parseIncludedFiles = false;
   }
 }
 
-void Parser::parseIncludedFiles()
+void Parser::clearIncludedGroupElements()
 {
-#ifdef DEBUG_PARSER
-  kdDebug(24000) << "parseIncludedFiles" << endl;
-#endif
-  IncludedGroupElementsMap::Iterator includedMapIt;
   uint listCount;
+  IncludedGroupElementsMap::Iterator includedMapIt;
   for (includedMapIt = includedMap.begin(); includedMapIt != includedMap.end(); ++includedMapIt)
   {
     IncludedGroupElements::Iterator elementsIt;
@@ -1350,12 +1351,24 @@ void Parser::parseIncludedFiles()
         listCount = it.data().count();
         for (uint i = 0 ; i < listCount; i++)
         {
-          delete it.data()[i].node;
+          GroupElement *groupElement = it.data()[i];
+          groupElement->node->tag->write()->userTagList.remove(groupElement->node->tag->name.lower());
+          delete it.data()[i]->node;
+          delete it.data()[i];
         }
       }
     }
   }
   includedMap.clear();
+}
+
+void Parser::parseIncludedFiles()
+{
+#ifdef DEBUG_PARSER
+  kdDebug(24000) << "parseIncludedFiles" << endl;
+#endif
+  clearIncludedGroupElements();
+  uint listCount;
   if (write->url().isLocalFile())
   {
     listCount = ParserCommon::includedFiles.count();
@@ -1363,12 +1376,25 @@ void Parser::parseIncludedFiles()
     {
       parseIncludedFile(ParserCommon::includedFiles[i], ParserCommon::includedFilesDTD.at(i));
     }
+    if (listCount > 0)
+      m_parseIncludedFiles = false;
   }
   emit rebuildStructureTree(true);
 }
 
+//structure used to temporary store the position of the groupelements in the searchFor
+//included file as a string
+struct GroupElementPosition{
+  GroupElement *element;
+  int startPos;
+  int endPos;
+};
+
 void Parser::parseIncludedFile(const QString& fileName, const DTDStruct *dtd)
 {
+#ifdef DEBUG_PARSER
+  kdDebug(24000) << "parseIncludedFile: " << fileName << endl;
+#endif  
   StructTreeGroup group;
   QString content;
   QFile file(fileName);
@@ -1376,7 +1402,13 @@ void Parser::parseIncludedFile(const QString& fileName, const DTDStruct *dtd)
   {
     IncludedGroupElements *elements = &includedMap[fileName];
     QTextStream str(&file);
-    str.setEncoding(QTextStream::UnicodeUTF8);
+    QString encoding;
+    KTextEditor::EncodingInterface* encodingIf = dynamic_cast<KTextEditor::EncodingInterface*>(write->doc());
+    if (encodingIf)
+      encoding = encodingIf->encoding();
+    if (encoding.isEmpty())
+      encoding = "utf8";  //final fallback
+    str.setCodec(QTextCodec::codecForName(encoding));
     content = str.read();
     file.close();
     if (dtd->specialAreas.count())
@@ -1404,87 +1436,154 @@ void Parser::parseIncludedFile(const QString& fileName, const DTDStruct *dtd)
             lastAreaPos = areaPos2 + 1;
           }
           QuantaCommon::removeCommentsAndQuotes(foundStr, dtd);
-          //find the structure blocks and remove the text inside them
+          
+          //gather the starting position of structures
+          QValueList<uint> structPositions;
           int structPos = 0;
           while (structPos !=-1)
           {
             structPos = foundStr.find(dtd->structBeginStr, structPos);
             if (structPos != -1)
             {
-              int openNum = 1;
-              int pos = structPos + dtd->structBeginStr.length();
-              while (openNum !=0 && pos != -1)
-              {
-                pos = dtd->structRx.search(foundStr, pos);
-                if (pos != -1)
-                {
-                  if (dtd->structRx.cap() == dtd->structBeginStr)
-                      openNum++;
-                  else
-                      openNum--;
-                  pos++;
-                }
-              }
-              if (pos == -1)
-                  pos = foundStr.length();
-              QString spaces;
-              spaces.fill(' ', pos - structPos + 1);
-              foundStr.replace(structPos, pos - structPos + 1, spaces);
-              int openBracketPos = foundStr.findRev(dtd->structKeywordsRx, structPos);
-              openBracketPos = foundStr.find('(', openBracketPos);
-              openNum = 1;
-              if (openBracketPos != -1)
-              {
-                int closeBracketPos = openBracketPos;
-                while (closeBracketPos < structPos && openNum !=0)
-                {
-                  if (foundStr[closeBracketPos] == '(')
-                      openNum++;
-                  if (foundStr[closeBracketPos] == ')')
-                      openNum--;
-                  closeBracketPos++;
-                }
-                spaces.fill(' ', closeBracketPos - openBracketPos);
-                foundStr.replace(openBracketPos, closeBracketPos - openBracketPos, spaces);
-              }
-
-              structPos =  pos + 1;
+              structPositions.append(structPos);
+              structPos += dtd->structBeginStr.length();
             }
-          }
-          QValueList<StructTreeGroup>::ConstIterator it;
-          for (it = dtd->structTreeGroups.begin(); it != dtd->structTreeGroups.end(); ++it)
+          }          
+          
+          QValueList<GroupElementPosition> gPositions;
+          //go through the list of found structures and search for groups
+          int structStartPosition = 0; //from where to start the group search. This is before the structure begin position
+          QString savedStr = foundStr;
+          for (uint i = 0; i < structPositions.count(); i++)
           {
-            group = *it;
-            int pos = 0;
-            while (pos != -1)
+            foundStr = savedStr;
+            uint structBeginPos = structPositions[i];
+            structPos = structBeginPos;
+            int openNum = 1;
+            int pos = structPos + dtd->structBeginStr.length();
+            //find the corresponding structure closing string
+            while (openNum !=0 && pos != -1)
             {
-              pos = group.searchRx.search(foundStr, pos);
+              pos = dtd->structRx.search(foundStr, pos);
               if (pos != -1)
               {
-                QString s = content.mid(areaPos + pos, group.searchRx.matchedLength());
-                pos += s.length();
-                s.remove(group.clearRx);
-                if (!(*elements)[group.name].contains(s))
-                {
-                  Tag *tag = new Tag();
-                  tag->name = s;
-                  tag->setDtd(dtd);
-                  QString s2 = content.left(areaPos + pos);
-                  int newLineNum = s2.contains('\n');
-                  int tmpCol = s2.length() - s2.findRev('\n') - 1;
-                  tag->setTagPosition(newLineNum, tmpCol - s.length(), newLineNum, tmpCol);
-                  Node *node = new Node(0L);
-                  node->tag = tag;
-                  node->fileName = fileName;
-                  GroupElement groupElement;
-                  groupElement.node = node;
-                  groupElement.parentNode = 0L;
-                  GroupElementList *groupElementList = &(*elements)[group.name][s];
-                  groupElementList->append(groupElement);
-                }
+                if (dtd->structRx.cap() == dtd->structBeginStr)
+                    openNum++;
+                else
+                    openNum--;
+                pos++;
               }
             }
-          } //for
+            if (pos == -1)
+                pos = foundStr.length();
+            int structEndPos = pos;
+            foundStr = foundStr.left(pos);
+            QString spaces;
+            spaces.fill(' ', pos - structPos + 1);
+            foundStr.replace(structPos, pos - structPos + 1, spaces);
+            
+            //FIXME: This code replaces the content between ( ) with
+            //empty spaces. This is quite PHP (or functions) //specific, and it's done in order to not find variables
+            //declared as function arguments. A generic way is needed
+            //to exclude unwanted areas.
+            int openBracketPos = foundStr.findRev(dtd->structKeywordsRx, structPos);
+            openBracketPos = foundStr.find('(', openBracketPos);
+            openNum = 1;
+            if (openBracketPos != -1)
+            {
+              openBracketPos++;
+              int closeBracketPos = openBracketPos;
+              while (closeBracketPos < structPos && openNum !=0)
+              {
+                if (foundStr[closeBracketPos] == '(')
+                    openNum++;
+                if (foundStr[closeBracketPos] == ')')
+                    openNum--;
+                closeBracketPos++;
+              }
+              closeBracketPos--;
+              spaces.fill(' ', closeBracketPos - openBracketPos);
+              foundStr.replace(openBracketPos, closeBracketPos - openBracketPos, spaces);
+            }
+
+            //now check which groups are present in this area
+            structPos =  pos + 1;
+            QValueList<StructTreeGroup>::ConstIterator it;
+            for (it = dtd->structTreeGroups.begin(); it != dtd->structTreeGroups.end(); ++it)
+            {
+              group = *it;
+              if (!group.hasDefinitionRx)
+                continue;
+              int pos = structStartPosition;
+              while (pos != -1)
+              {
+                pos = group.definitionRx.search(foundStr, pos);
+                if (pos != -1)
+                {
+                  int l;
+                  QString ss = group.definitionRx.cap();
+                  if (group.definitionRx.pos(1) > pos)
+                  {
+                    pos = group.definitionRx.pos(1);
+                    l = group.definitionRx.cap(1).length();
+                    ss = group.definitionRx.cap(1);
+                  }
+                  else
+                  {
+                    l = group.definitionRx.cap().length();
+                  }
+                  QString s = content.mid(areaPos + pos, l);
+                  pos += l;
+                  if (!(*elements)[group.name].contains(s))
+                  {
+                    Tag *tag = new Tag();
+                    tag->name = s;
+                    tag->setDtd(dtd);
+                    tag->setWrite(write);
+                    QString s2 = content.left(areaPos + pos);
+                    int newLineNum = s2.contains('\n');
+                    int tmpCol = s2.length() - s2.findRev('\n') - 1;
+                    tag->setTagPosition(newLineNum, tmpCol - s.length(), newLineNum, tmpCol);
+                    Node *node = new Node(0L);
+                    node->tag = tag;
+                    node->fileName = fileName;
+                    GroupElement *groupElement = new GroupElement;
+                    groupElement->node = node;
+                    groupElement->parentNode = 0L;
+                    int minPos = areaPos + pos + 1;
+                    for (QValueList<GroupElementPosition>::Iterator gPosIt = gPositions.begin(); gPosIt != gPositions.end(); ++gPosIt)
+                    {
+                      GroupElementPosition gPos = (*gPosIt);
+                      if ( (areaPos + pos > gPos.startPos) && (areaPos + pos < gPos.endPos) && (gPos.startPos < minPos))
+                      {
+                        groupElement->parentNode = gPos.element->node;          
+                        minPos = gPos.startPos; 
+                      }
+                    }
+                    GroupElementList *groupElementList = &(*elements)[group.name][s];
+                    groupElementList->append(groupElement);
+                    
+                    GroupElementPosition gPos;
+                    gPos.startPos = areaPos + pos;
+                    gPos.endPos = structEndPos;
+                    gPos.element = groupElement;
+                    gPositions.append(gPos);
+                    
+                    if (group.appendToTags)
+                    {
+                      QTag *qTag = new QTag();
+                      qTag->setName(s.left(s.find('(')));
+                      qTag->className = "";
+                      if (groupElement->parentNode)
+                        qTag->className = groupElement->parentNode->tag->name;
+                      write->userTagList.insert(s.lower(), qTag);
+                    }
+                  }
+                }
+              }
+            } //for
+            structStartPosition = structBeginPos + 1;
+          }
         } //if (areaPos != -1)
       }// while (areaPos != -1)
     }
@@ -1499,6 +1598,20 @@ void Parser::slotIncludedFileChanged(const QString& fileName)
     const DTDStruct *dtd = ParserCommon::includedFilesDTD.at(pos);
     if (dtd)
     {
+      IncludedGroupElements::Iterator elementsIt;
+      for (elementsIt = includedMap[fileName].begin(); elementsIt != includedMap[fileName].end(); ++elementsIt)
+      {
+        GroupElementMapList::Iterator it;
+        for (it = elementsIt.data().begin(); it != elementsIt.data().end(); ++it)
+        {
+          uint listCount = it.data().count();
+          for (uint i = 0 ; i < listCount; i++)
+          {
+            delete it.data()[i]->node;
+            delete it.data()[i];
+          }
+        }
+      }
       includedMap[fileName].clear();
       parseIncludedFile(fileName, dtd);
     }
@@ -1526,8 +1639,17 @@ void Parser::parseForXMLGroup(Node *node)
     title = title.left(title.length()-3);
     title.remove('\n');
     newTag->name = title;
-    node->groupTag = newTag;
-    node->group = const_cast<XMLStructGroup*>(&(xmlGroupIt.data()));
+
+    GroupElement *groupElement = new GroupElement;
+    groupElement->deleted = false;
+    groupElement->tag = newTag;
+    groupElement->node = node;
+    groupElement->parentNode = 0L;
+    groupElement->global = true;
+    groupElement->group = const_cast<XMLStructGroup*>(&(xmlGroupIt.data()));
+    node->m_groupElements.append(groupElement);
+    GroupElementList* groupElementList = & (globalGroupMap[group.name + "|" + title]);
+    groupElementList->append(groupElement);
   }
 }
 
@@ -1607,9 +1729,15 @@ void Parser::slotParseInDetail()
   m_saParser->parseInDetail(false);
 }
 
+void Parser::synchParseInDetail()
+{
+  m_saParser->parseInDetail(false);
+}
+
 void Parser::setSAParserEnabled(bool enabled)
 {
   m_saParser->setParsingEnabled(enabled);
+  //kapp->processEvents(QEventLoop::ExcludeUserInput | QEventLoop::ExcludeSocketNotifiers); //this makes sure that the parsing is really disabled
 }
 
 #include "parser.moc"
