@@ -98,7 +98,7 @@ void  ProjectUpload::initProjectInfo()
   baseUrl = new KURL();
 
 //  list->setMultiSelection(true);
-  QDomDocument *dom = m_project->dom();
+  QDomDocument *dom = m_project->sessionDom();
   m_profilesNode = dom->firstChild().firstChild().namedItem("uploadprofiles");
   if (m_profilesNode.isNull()) //compat code, remove when upgrade from 3.2 is not supported
   {
@@ -150,6 +150,7 @@ void  ProjectUpload::initProjectInfo()
 /** No descriptions */
 void ProjectUpload::slotBuildTree()
 {
+ emit eventHappened("upload_requested", m_project->projectBaseURL().url(), "");
  loadRemoteUploadInfo();
  KIO::UDSEntry entry;
  QString strUrl = QuantaCommon::qUrl(startUrl);
@@ -171,7 +172,7 @@ void ProjectUpload::slotBuildTree()
 
  if (startUrl.isEmpty() || strUrl.endsWith("/")) //upload a folder 
  {
-   projectDirFiles = QExtFileInfo::allFilesDetailed(u, "*");
+   projectDirFiles = QExtFileInfo::allFilesDetailed(u, "*", this);
  } else
  {
    projectDirFiles.insert(u.url(), new KFileItem(KFileItem::Unknown, KFileItem::Unknown, u, true));
@@ -368,7 +369,7 @@ void ProjectUpload::startUpload()
     buttonUpload->setEnabled(false);
     KURL u = *baseUrl;
     u.setPath(u.protocol() == "file" ? "/" : "");
-    if (QExtFileInfo::exists(u))
+    if (QExtFileInfo::exists(u, this))
     {
       upload();
       return;
@@ -431,7 +432,7 @@ void ProjectUpload::upload()
       if ( !madeDirs.contains(dir) )
       {
         madeDirs.append( dir );
-        if (!QExtFileInfo::createDir(dir))
+        if (!QExtFileInfo::createDir(dir, this))
         {
           QuantaCommon::dirCreationError(this, KURL( dir.prettyURL(0, KURL::StripFileProtocol) ));
           buttonUpload->setEnabled(true);
@@ -626,8 +627,8 @@ void ProjectUpload::slotRemoveProfile()
    } else
    {
       QString profileName = comboProfile->currentText();
-      if (KMessageBox::warningYesNo(this, i18n("<qt>Do you really want to remove the <b>%1</b> upload profile?</qt>").arg(profileName),
-                                   i18n("Profile Removal")) == KMessageBox::Yes)
+      if (KMessageBox::warningContinueCancel(this, i18n("<qt>Do you really want to remove the <b>%1</b> upload profile?</qt>").arg(profileName),
+                                   i18n("Profile Removal"), KStdGuiItem::del()) == KMessageBox::Continue)
        {
           m_profilesNode.removeChild(m_currentProfileElement);
           int idx = comboProfile->currentItem();
@@ -699,7 +700,7 @@ void ProjectUpload::readProfileDlg(UploadProfileDlgS *profileDlg)
   m_currentProfileElement.setAttribute("name", profileDlg->lineProfileName->text());
   m_currentProfileElement.setAttribute("remote_host", profileDlg->lineHost->text());
   m_currentProfileElement.setAttribute("user", profileDlg->lineUser->text());
-  m_currentProfileElement.setAttribute("remote_path", profileDlg->linePath->text());
+  m_currentProfileElement.setAttribute("remote_path", path);
   m_currentProfileElement.setAttribute("remote_port", profileDlg->port->text());
   m_currentProfileElement.setAttribute("remote_protocol", profileDlg->comboProtocol->currentText());
   QString passwd = QString(profileDlg->linePasswd->password());
@@ -744,8 +745,8 @@ void ProjectUpload::reject()
   if (uploadInProgress && !m_profilesOnly)
   {
     suspendUpload = true;
-    if (KMessageBox::questionYesNo(this,i18n("Do you really want to cancel the upload?"),
-                                   i18n("Cancel Upload")) == KMessageBox::No)
+    if (KMessageBox::questionYesNo(this,i18n("Do you really want to abort the upload?"),
+                                   i18n("Abort Upload"), i18n("Abort"), KStdGuiItem::cancel()) == KMessageBox::No)
     {
       suspendUpload = false;
       emit uploadNext();
