@@ -180,7 +180,7 @@ void QuantaNetAccess::checkProjectInsert(const KURL& target, QWidget* window, bo
     {
       QString nice = QExtFileInfo::toRelative(saveUrl, baseURL).path();
       nice = KStringHandler::lsqueeze(nice, 60);
-      if ( KMessageBox::Yes != KMessageBox::questionYesNo(window, i18n("<qt>Do you want to add <br><b>%1</b><br> to the project?</qt>").arg(nice), i18n("Add to Project"), KStdGuiItem::yes(), KStdGuiItem::no(), "AddToProject") )
+      if ( KMessageBox::Yes != KMessageBox::questionYesNo(window, i18n("<qt>Do you want to add <br><b>%1</b><br> to the project?</qt>").arg(nice), i18n("Add to Project"), KStdGuiItem::add(), i18n("Do Not Add"), "AddToProject") )
       {
         return;
       }
@@ -205,7 +205,7 @@ bool QuantaNetAccess::checkProjectRemove(const KURL& src, QWidget* window, bool
     {
       QString nice = QExtFileInfo::toRelative(url, baseURL).path();
       nice = KStringHandler::lsqueeze(nice, 60);
-      if ( KMessageBox::Continue != KMessageBox::warningContinueCancel(window, i18n("<qt>Do you really want to remove <br><b>%1</b><br> from the project?</qt>").arg(nice), i18n("Remove From Project"), KStdGuiItem::del(), "RemoveFromProject") )
+      if ( KMessageBox::Continue != KMessageBox::warningContinueCancel(window, i18n("<qt>Do you really want to remove <br><b>%1</b><br> from the project?</qt>").arg(nice), i18n("Remove From Project"), KStdGuiItem::remove(), "RemoveFromProject") )
       {
         return false;
       }
